%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Spectrum Analysis with MATLAB Implementation     %
%                                                      %
% Author: M.Sc. Eng. Hristo Zhivomirov       08/24/15  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Xamp, Xph, f] = spectrum(x, w, nfft, fs)

% function: [Xamp, Xph, f] = spectrum(x, w, nfft, fs)
% x - signal in the time domain
% w - window function in the time domain
% nfft - size of the fft
% fs - sampling frequency, Hz
% Xamp - amplitude spectrum, dB
% Xph - phase spectrum, degrees
% f - frequency vector, Hz

% represent x as as column-vector
x = x(:);

% represent w as column-vector
w = w(:);

% length of the signal
N = length(x);

% windowing
x = x.*w;

% take fft of x
fftx = fft(x, nfft);

% calculate the number of unique fft points
NumUniquePts = ceil((nfft+1)/2);

% fft is symmetric, throw away the second half
fftx = fftx(1:NumUniquePts);

% define the coherent amplification of the window
K = sum(w)/N;

% take the magnitude of fft(x) and scale it, so not to be a
% function of the length of x and window coherent amplification
Xamp = abs(fftx)/N/K;

% correction of the amplitude spectrum
if rem(nfft, 2)     % odd nfft excludes Nyquist point
    Xamp(2:end) = Xamp(2:end)*2;
else                % even nfft includes Nyquist point
    Xamp(2:end-1) = Xamp(2:end-1)*2;
end

% convert amplitude spectrum to dB
Xamp = 20*log10(Xamp);

% phase spectrum
Xph = angle(fftx);

% convert phase spectrum to degrees
Xph = rad2deg(Xph);

% generate frequency vector with NumUniquePts points
f = (0:NumUniquePts-1)*fs/nfft;

end