%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Spectrum Analysis with MATLAB Implementation     %
%                                                      %
% Author: M.Sc. Eng. Hristo Zhivomirov       06/10/13  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Xamp, Xph, f] = spectrum(x, fs)

% function: [Xamp, Xph, f] = spectrum(x, fs)
% x - signal in the time domain
% fs - sampling frequency, Hz
% Xamp - amplitude spectrum, dB
% Xph - phase spectrum, degrees
% f - frequency vector, Hz

% represent x as column-vector
x = x(:);

% length of the signal
N = length(x);

% windowing
win = hanning(N, 'periodic');
x = x.*win;

% take fft of x
fftx = fft(x);

% calculate the number of unique fft points
NumUniquePts = ceil((N+1)/2);

% fft is symmetric, throw away the second half
fftx = fftx(1:NumUniquePts);

% define the coherent amplification of the window
K = sum(win)/N;

% take the magnitude of fft(x) and scale it, so not to be a
% function of the length of x and window coherent amplification
Xamp = abs(fftx)/N/K;

% correction of the amplitude spectrum
if rem(N, 2)        % odd N excludes Nyquist point
    Xamp(2:end) = Xamp(2:end)*2;
else                % even N includes Nyquist point
    Xamp(2:end-1) = Xamp(2:end-1)*2;
end

% convert amplitude spectrum to dB
Xamp = 20*log10(Xamp);

% phase spectrum
Xph = angle(fftx);

% convert phase spectrum to degrees
Xph = rad2deg(Xph);

% generate frequency vector with NumUniquePts points
f = (0:NumUniquePts-1)*fs/N;

end